package taxonomyLookupService;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

import javax.servlet.http.HttpServletResponse;

/**
 * An object to parse and hold the data scrapped from the NCBI flat file response.
 */
public class NCBIFlatFile {

	/**
     * URL for NCBI Taxonomy Database lookup
     * (Access to the NCBI taxonomic data)
     */
    private static final String NCBI_TAXONOMY_DATABASE_QUERY_URL = "http://www.ncbi.nlm.nih.gov/Taxonomy/TaxIdentifier/tax_identifier.cgi";

	/**
     * URL path to the NCBI Taxonomy Database with query string stub.
     * 
     * RESTful URL to a taxon; specialized to a taxon once the id value is appended; examples:
     * 		http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?lvl=0&id=3880
     * 		http://purl.org/taxa/ncbi/3880
     */
    // private static final String NCBI_TAXONOMY_RECORD_URL_STUB = "http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?lvl=0&id=";
    private static final String NCBI_TAXONOMY_RECORD_URL_STUB = "http://purl.org/taxa/ncbi/";

    /**
     * Local variables for specific NCBI taxon data
     */
	private StringBuilder scientificNameStrBldr;
	private StringBuilder taxonomyIDStrBldr;
	private StringBuilder commonNameStrBldr;
	
	private BufferedReader bufferedReader;
	
	// Constructor
	/**
     * Make a connection to NCBI and set up the flat file returned for parsing.
     * 
	 * @param lookupStr the lookup key to send to NCBI; e.g., a taxonomy ID or a taxon common name
	 * @throws IOException on a network failure when making the connection to NCBI
     */
    public NCBIFlatFile(String lookupStr) throws IOException {

    	HttpURLConnection ncbiConnection = null;
    	
    	try {

    		// send the lookup key to NCBI to get its mapping to a scientific record
    		ncbiConnection = sendToNCBI(lookupStr);
    		initialize(ncbiConnection.getInputStream());

    		// if it parses as an integer, consider it a NCBI Taxonomy ID, otherwise as a common name
    		try {
    			Integer.parseInt(lookupStr);
    			lookupTaxonomyID(lookupStr);
    		} catch ( NumberFormatException nfe ) {
    			lookupCommonName(lookupStr);
    		}

    	} catch ( IOException ioe ) {
    		throw ioe;
    	} finally {    // no matter what, clean up
    		
    		close();

    		if ( ncbiConnection != null ) {
    			ncbiConnection.disconnect();
    		}
    	}
    }

    
    /**
     * Initialize and allocate
     * 
     * @param inputStream the stream of data from an NCBI open connection
     */
    private void initialize(InputStream inputStream) {

		scientificNameStrBldr = new StringBuilder();
		taxonomyIDStrBldr = new StringBuilder();
		commonNameStrBldr = new StringBuilder();

		bufferedReader = new BufferedReader(new InputStreamReader(inputStream));

	}
    
    /**
     * Given a NCBI taxonomy ID as input, parse the flat file to set local variables.
     * 
     * @param taxonomyIDValue taxonomy ID as key to parsing flat file
     * @throws IOException on read buffer error
     */
	void lookupTaxonomyID(String taxonomyIDValue) throws IOException {

		// the user input taxonomyIDValue may be the NCBI secondary taxid;
		// Ultimately, taxid will be discarded and the parseTaxonomicFlatText()
		// method will set the (primary) NCBI taxonomyID. This is what
		// is used for the recordURL
		StringBuilder taxid = new StringBuilder(taxonomyIDValue);
		parseTaxonomicFlatText(bufferedReader, taxid, taxonomyIDStrBldr, scientificNameStrBldr);

	}

	 /**
     * Given a common (taxon) name as input, parse the flat file to set local variables.
     * 
     * @param commonNameValue common (taxon) name (e.g. dog, cat) as key to parsing flat file
     * @throws IOException on read buffer error
     */
	void lookupCommonName(String commonNameValue) throws IOException {
		commonNameStrBldr = new StringBuilder(commonNameValue);
		parseTaxonomicFlatText(bufferedReader, commonNameStrBldr, scientificNameStrBldr, taxonomyIDStrBldr);
	}

	/**
	 * Get the scientific name for the taxon.
	 * @return NCBI scientific name (may be empty)
	 */
	String getScientificName() {
		// scientificName may be empty if the 'commonName' was the NCBI "preferred" (i.e., scientific) name
		return scientificNameStrBldr.toString();
	}

	/**
	 * Get the one fo the common names for the taxon.
	 * @return Common (vernacular) name (may be empty)
	 */
	String getCommonName() {
		return commonNameStrBldr.toString();
	}

	/**
	 * Get the NCBI taxonomy ID (unique for each taxon)
	 * @return NCBI taxonomy ID
	 */
	String getTaxonomyID() {
		return taxonomyIDStrBldr.toString();
	}

	/**
	 * Get the URL to the NCBI record. May be of the NCBI domain or a redirect such as a persistent URL (PURL).
	 * @return URL to the NCBI record
	 */
	String getRecordURL() {
		return NCBI_TAXONOMY_RECORD_URL_STUB + getTaxonomyID();
	}

	/**
	 * Close the read buffer
	 */
	void close() {
		if ( bufferedReader != null ) {
			try {
				bufferedReader.close();
			} catch (IOException e) {
				;  // consume
			}
		}
	}
	
    
    /**
     * POST a database retrieval request to NCBI and return the opened connection.
     * Caller should close the connection when done.
     * 
     * @param lookupKey the term to lookup in the database
     * @return an open connection on success
     * @throws IOException on any failure
     */
    private HttpURLConnection sendToNCBI(String lookupKey) throws IOException {
    	
    	HttpURLConnection ncbiConnection = null;
    	
		try {

    		// Open a connection to the NCBI_TAXONOMY_DATABASE_QUERY_URL
    		URL ncbiURL = new URL(NCBI_TAXONOMY_DATABASE_QUERY_URL);
    		ncbiConnection = (HttpURLConnection) ncbiURL.openConnection();

    		// Set connection parameters
    		ncbiConnection.setRequestMethod("POST");
    		ncbiConnection.setDoOutput(true);	// for writing the POST BOBY to the connection

    		// Set up to write the input request to the NCBI URL
    		PrintWriter writer = new PrintWriter(new OutputStreamWriter(ncbiConnection.getOutputStream()));

    		// Send the lookup key in the <TEXTAREA> called "tax";
    		// See the source on the NCBI_TAXONOMY_DATABASE_QUERY_URL page for parameter names and expected values
    		writer.write("tax=" + URLEncoder.encode(lookupKey, "UTF-8"));

    		// Add an '&' between parameters
    		writer.write("&");
    		
    		// Choose the "Save in File" submit button so that minimal information is returned
    		writer.write("button=" + URLEncoder.encode("Save in file", "UTF-8"));
    		
    		// We're done writing to the ncbiURL
    		writer.close();

    		// Check the status of the POST - if it is anything other than OK, bail
    		int responseCode = ncbiConnection.getResponseCode();
    		if ( responseCode != HttpServletResponse.SC_OK ) {
    			String responseMessage = ncbiConnection.getResponseMessage();
    			responseMessage = responseMessage != null ? " : " + responseMessage : "";
    			throw new Exception(responseCode + responseMessage);
    		}

    	} catch ( Throwable t ) {
    		
    		if ( ncbiConnection != null ) {
    			ncbiConnection.disconnect();
    		}
    		
			throw new IOException(t.getMessage());

    	}
    	
		return ncbiConnection;
    	
    }
    
    
	 /**
     * Parses the NCBI flat text format and writes the results into string buffers.
     *
     * The flat file format for NCBI Taxonomy data returned from
     * NCBI_TAXONOMY_DATABASE_QUERY_URL is explained at the URL.
     * 
     * If parsing the results of a search on the commonName or scientificName,
     * then the signature field order is:
     * 	field1 = commonName
     * 	field2 = scientificName
     *	field3 = taxonomyID
     * 
     * If parsing the results of a search on the taxonomyID, then
     * the signature field order is:
     * 	field1 = taxid
     *	field2 = taxonomyID
     *	field3 = scientificName
     * 
     * The NCBI file format uses "codes" to indicate the result state.
     * For the commonName or scientificName search, the NCBI file format is:
     * 
     *    code	|	name	|	preferred name	|	taxid
     *    1	|	Medicago truncatula	|	|	3880
     *    2	|	barrel medic	|	Medicago truncatula	|	3880
     *    3	|	garbage	|	|
     *    
     *    There is always the header line; fields are separated by both
     *    tabs and a pipe ('|').
     *    
     *    code 1 = the search name is the primary or "preferred name".
     *    	The "preferred name" field may or may not have a value (depends on
     *      capitalization).  The taxid field will have a value.
     *    
     *    code 2 = the search name is a secondary name (common name,
     *    	synonym, etc.).  There will be a value in the preferred name field and
     *    	the taxid field.
     *    
     *    code 3 = no match; no values in the preferred name nor taxid fields.
     *    
     *    Some search names may result in multiple entries, each with different taxids.
     *    In these cases, a '+' will be appended to the code (e.g., 2+)
     *    
     *    This routine ignores possible multiple matches.
     *    
     *    If a taxid exists, it is set.  Conditional on that, if a preferredName exists,
     *    then the scientificName is set to that value.
     *  
     *    For the taxonomyID search, the NCBI file format is:
     *    
     *    code	|	taxid	|	primary taxid	|	taxname
     *    1	|	2	|	2	|	Bacteria
     *    2	|	12	|	74109	|	Photobacterium profundum
     *    3	|	1
     *    4	|	3
     *   
     *    code = 1 means "the incoming taxid is our primary id for a taxon in our database"
     *    
     *    code = 2 means "the incoming taxid is a secondary id for a taxon in our database"
     *    
     *    code = 3 means "the incoming taxid is not yet associated with a public sequence entry"
     *    
     *    code = 4 means "the incoming taxid is not found in our database"
     *    
     *    Unfortunately, searches on taxid do not return commonName information. 
     *    
     *    @param bufferedReader NCBI flat file to parse
     *    @param field1 depends on file: commonName or taxid
     *    @param field2 depends on file: scientificName or taxonomyID
     *    @param field3 depends on file: taxonomyID or scientificName
     *    @return if taxid is set (i.e., codes 1 or 2), the method returns 0 (success); otherwise 1 (failure).
     *    @throws IOException on read buffer error
     */
    private int parseTaxonomicFlatText(BufferedReader bufferedReader, StringBuilder field1, StringBuilder field2, StringBuilder field3) throws IOException {


    	int		returnCode = 1;	// default is to fail
    	String	inputLine;		// read a line at a time

    	while ( (inputLine = bufferedReader.readLine()) != null ) {

    		// split the line into at most 4 fields
    		String[] splitLine = inputLine.split("[|]",4);	// split at the pipe '|' char

    		if ( splitLine.length == 4 ) {

    			String code = splitLine[0].trim(),	// get the 'code' field
    			       tmpField1,					// commonName or taxid
    			       tmpField2;					// scientificName or taxonomyID
    			
    			if ( code.equals("1") || code.equals("2") ) {	// a match

    				tmpField1 = splitLine[1].trim();	// default
    				tmpField2 = splitLine[2].trim();
    				
    				if ( tmpField2.equals("") ) {		// scientificName is blank when search name is primary name
    					tmpField2 = tmpField1;
    					tmpField1 = "";
    				}
    			} else {
    				continue;	// done (failed) for this line
    			}
    			
    			// set values
    			// setting commonName or taxid
    			field1.delete(0,field1.length());
    			field1.append(tmpField1);
    			
    			// setting scientificName or taxonomuyID
				field2.delete(0,field2.length());
				field2.append(tmpField2);
				
				// setting taxonomyID or scientificName
				field3.delete(0,field3.length());
				field3.append(splitLine[3].trim());

    			// set return code to success
    			returnCode = 0;

    			// leave the loop; we're done
    			break;
    		}
    	}

    	return returnCode;

    }

}
