/**
 * Copyright (c) 2010, iPlant Collaborative, University of Arizona, Cold Spring Harbor Laboratories, University of Texas at Austin
 * This software is licensed under the CC-GNU GPL version 2.0 or later.
 * License: http://creativecommons.org/licenses/GPL/2.0/
 */
package taxonomyLookupService;

import info.sswap.api.model.*;
import info.sswap.api.servlet.MapsTo;

import taxonomyLookupService.Ontologies;

/**
 * This servlet class handles requests for the TaxonomyLookupService.
 * 
 * Override the single method handleRequest() to "do" what the web service does.
 * 
 */
public class TaxonomyLookupService extends MapsTo {
	
    /**
	 * Default
	 */
	private static final long serialVersionUID = 1L;

    /**
     * Instance variables of we use
     */
    private TaxonomyRecord taxonomyRecord;
    private SSWAPPredicate commonName;
    private SSWAPPredicate taxonomyID;
    
    /**
     * Types and predicates created in the Resource Invocation Graph (RIG) document.
     * 
     * @param rig document within which to get/create the types and predicates
     */
    @Override
	protected void initializeRequest(RIG rig) {
    	
    	// if we need to check service parameters we could start here
    	//SSWAPResource translatedResource = rig.getTranslatedResource();
    	
    	taxonomyRecord = new TaxonomyRecord(rig);
    	commonName = rig.getPredicate(Ontologies.NCBITaxonomyRecord.commonNameURI);
    	taxonomyID = rig.getPredicate(Ontologies.NCBITaxonomyRecord.taxonomyIDURI);
    }
    
    /**
     * Any post-request cleanup can be done here.
     */
    @Override
	protected void finalizeRequest(Exception e) { }

	/**
	 * This method will be called automatically if, and only if, the client sends
	 * this web service a valid RIG (Resource Invocation Graph); i.e., a graph
	 * that is logically consistent with the statements made in this service's
	 * RDG (Resource Description Graph). If a client sends a request that is not
	 * serviceable by the web service as expressed in its RDG, then
	 * AbstractSSWAPServlet handles all the details and response to the client;
	 * this method is not even entered.
	 * 
	 * If the client sends an RIG that is logically consistent with the RDG,
	 * then <code>initializeRequest(RIG rig)</code> is called with the client's
	 * RIG as the variable 'rig'. Then this method is called for each
	 * subject. The subject is passed through a "semantic translator" that maps
	 * vocabularies of the RIG into terms recognized by the RDG. Finally,
	 * <code>finalizeRequest(Exception e)</code> is called as the last method
	 * before the putative RRG (Resource Response Graph) is returned for further
	 * processing.
	 * 
	 * Before returning a RRG response graph to the client, AbstractSSWAPServlet
	 * checks to see if the putative RRG is itself logically consistent with the
	 * RDG. If it is, it is returned to the client; if it is not, the client's
	 * original RIG is returned. Thus on either server-side success or (logical)
	 * error, the client always receives a logically valid response (that could
	 * be sent to other services on the web).
	 * 
	 * The idea is to think of semantic web services as not just web services
	 * with semantics added--web services that may work or fail. But to think of
	 * semantic web services as information agents on the web, which at any time
	 * may add more or less monotonic information to a graph of logical
	 * statements.
	 * 
	 * @param translatedSubject
	 *            the "current" SSWAPSubject after semantic translation
	 */
    @Override
	protected void mapsTo(SSWAPSubject translatedSubject) {

		NCBIFlatFile ncbiFlatFile = getNCBIFlatFile(translatedSubject);
		
		// operate (or pass) on all objects sent for a mapping
		if ( ncbiFlatFile != null ) {
			for ( SSWAPObject sswapObject : translatedSubject.getObjects() ) {
				taxonomyRecord.set(sswapObject, ncbiFlatFile);
			}
		}
		
		// to add new object(s), for example, because it may be unclear if
		// or how to modify existing objects, or because the nature of the
		// service is to return a 1:many mapping, use:
		
		// SSWAPObject sswapObject = assignObject(translatedSubject);
	
	}
    
    
    /**
     * Get a NCBI Flat File (results of a network call) based on the
     * values present on the sswapSubject.
     * 
     * @param sswapSubject source subject with lookup properties
     * @return a NCBI Flat File object suitable for self-parsing
     */
    private NCBIFlatFile getNCBIFlatFile(SSWAPSubject sswapSubject) {
    	
    	NCBIFlatFile ncbiFlatFile = null;

    	String lookupValue = getStrValue(sswapSubject,taxonomyID);
		if ( lookupValue == null || lookupValue.trim().isEmpty() ) {
			lookupValue = getStrValue(sswapSubject,commonName);
		}
		
		if ( lookupValue != null && ! lookupValue.trim().isEmpty() ) {
			
			// make the network call to NCBI and parse the results
			try {
				ncbiFlatFile = new NCBIFlatFile(lookupValue);
			} catch ( Exception e ) {
				; // fall through w/ null value
			}
		}
		
		return ncbiFlatFile; // may be null
    }
    
    
    /**
     * Returns the string value for a property instance on an individual.
     * If more than one property instance exists, only one is (arbitrarily) chosen.
     * 
     * @param sswapIndividual the individual with the property
     * @param propertyURI the URI identifying the property (predicate)
     * @return the value as a string; null on any failure
     */
    private String getStrValue(SSWAPIndividual sswapIndividual, SSWAPPredicate sswapPredicate) {
    	
		String value = null;
    	SSWAPProperty sswapProperty = sswapIndividual.getProperty(sswapPredicate);

		if ( sswapProperty != null ) {
			
			value = sswapProperty.getValue().asString();

			if ( value.isEmpty() ) {
				value = null;
			}
		}
		
		return value;
    }

}
