package taxonomyLookupService;

import info.sswap.api.model.*;

import taxonomyLookupService.Ontologies;

/**
 * Allows the setting of a SSWAPIndovidual (e.g., a SSWAPObject) to the
 * type and properties of a NCBI TaxonomyRecord.
 *
 */
public class TaxonomyRecord {

	/**
     * Instance variables of ontology terms we use
     */
	private SSWAPDocument sswapDocument;
    private SSWAPType TaxonomyRecordType;
    private SSWAPPredicate commonName;
    private SSWAPPredicate scientificName;
    private SSWAPPredicate taxonomyID;
    private SSWAPPredicate recordURL;
    
    /**
     * Constructor
     * @param sswapDocument document to associate the various types and predicates; e.g., RIG
     */
    TaxonomyRecord(SSWAPDocument sswapDocument) {
    	
    	this.sswapDocument = sswapDocument;
    	TaxonomyRecordType = sswapDocument.getType(Ontologies.NCBITaxonomyRecord.TaxonomyRecordURI);
    	commonName = sswapDocument.getPredicate(Ontologies.NCBITaxonomyRecord.commonNameURI);
    	scientificName = sswapDocument.getPredicate(Ontologies.NCBITaxonomyRecord.scientificNameURI);
    	taxonomyID = sswapDocument.getPredicate(Ontologies.NCBITaxonomyRecord.taxonomyIDURI);
    	recordURL = sswapDocument.getPredicate(Ontologies.NCBITaxonomyRecord.recordURLURI);

    }
    
    /**
     * Set the individual (usually a SSWAPObject) with the data in the NCBI flat file.
     * 
     * @param sswapIndividual individual which to set new types and properties
     * @param ncbiFlatFile source of data
     * @return true on success, false on failure
     */
	public boolean set(SSWAPIndividual sswapIndividual, NCBIFlatFile ncbiFlatFile) {
		
		try {

			SSWAPProperty taxonomyIDProperty = sswapIndividual.getProperty(taxonomyID);
			if ( taxonomyIDProperty != null ) {
				
				String taxonomyIDValue = taxonomyIDProperty.getValue().asString();
				if ( ! taxonomyIDValue.equals(ncbiFlatFile.getTaxonomyID()) ) {
					return false;	// had a taxonomyID, but not a match, so short-circuit and return
				}
			}
			
		} catch ( Exception e ) {
			;	// fall through to below
		}

		sswapIndividual.addType(TaxonomyRecordType);

		sswapIndividual.setProperty(commonName, ncbiFlatFile.getCommonName());
		sswapIndividual.setProperty(scientificName, ncbiFlatFile.getScientificName());
		sswapIndividual.setProperty(taxonomyID, ncbiFlatFile.getTaxonomyID());

		SSWAPLiteral recordURLLiteral = sswapDocument.createTypedLiteral(ncbiFlatFile.getRecordURL(),SSWAPDatatype.XSD.anyURI);
		sswapIndividual.setProperty(recordURL, recordURLLiteral);
		
		return true;
	}
	
}
