#!/bin/bash

# for every file in 'path' that ends in .json and is newer than
# a file by the same name but without the suffix, POST it to the SSWAP
# HTTP API for translation into OWL RDF/XML.
#
# Example:
#
# if myRDG.json is newer than myRDG, then send myRDG.json to the
# HTTP API and save/replace the contents of myRDG.
#
# Options:
#
# -c  path
#      on success, copy the resultant translated file to
#      the location relative to 'path'
# -f
#      force translation even if the .json file
#      is not newer.  This can be used to translate all files in
#      a directory regardless of time stamps.
#
# -p prefix
#      add (possibly multiple) prefix substitutions
#      example: -p myPrefix=http://mySite.org/ontologies/ -p somePre=http://someSite.org/
#
# -s apiServer
#      override the default API server;
#      example: -s http://localhost:8088/api
#
# To run with the current directory as the root path:
#
# 	mkTerms.sh .
#

usage()
{
	echo "Usage: `basename $0` [-f] [-c path] [-p prefix1=URI [-p prefix2=URI ] ... ] [-s API server] path ..." >&2
	exit 2;
}

error()
{
	echo `basename $0:` "$1" >&2
	exit 1;
}


# initialize
copyPathRoot=""
force=""
queryStr=""
apiServer=http://sswap.info/api
#apiServer=http://localhost:8088/api	# for making RIGs on localhost

# parse command line
while getopts "c:fp:s:" arg
do
	case "$arg" in
		c ) copyPathRoot="$OPTARG"
		    test -d "$copyPathRoot" || error "destination not a directory: $copyPathRoot" ;;
		f ) force=force ;;
		p ) test -z "$queryStr" && queryStr=?"$OPTARG" && continue
		    queryStr=${queryStr}\\\&"$OPTARG" ;;
		s ) apiServer="$OPTARG" ;;
		[?]) usage;;
	esac
done

shift $(($OPTIND - 1))
path="$*"

# sanity check on path argument
test -z "$path" && usage
test -r "$path" || error "cannot read '$path'"

# for every .json file ...
for jsonFile in `find -- "$path" -name '*.json'`
do

	# strip the suffix .json
	rdfName=`expr "$jsonFile" : '\(.*\)\.json$'`

	# if .json is newer, send to HTTP API
	if [ "$force" -o "$jsonFile" -nt "$rdfName" ]
	then
		# write to .err file; on success, rename to remove .err suffix
		responseCode=`curl -sw '%{http_code}\n' -d@${jsonFile} -o ${rdfName}.err ${apiServer}${queryStr}`

		if [ $responseCode -eq 200 ]
		then
			msg=OK
			mv "$rdfName".err "$rdfName"
			if [ "$copyPathRoot" ]
			then
				destPath="$copyPathRoot"/`dirname $jsonFile`
				cp -- "$jsonFile" "$rdfName" "$destPath"
			fi
		else	
			msg=Failed
		fi
		echo $responseCode $msg : $jsonFile
	fi
	
done

exit 0

